# -*- coding: utf-8 -*-
from .base import ListSubResource


def subresources(subresource_names):
    def _decorated(klass):
        orig_get = klass.get

        def get_wrapper(self, id):
            obj = orig_get(self, id)
            for subresource in subresource_names:
                setattr(
                    obj,
                    subresource,
                    ListSubResource(self, id, subresource)
                )
            return obj

        klass.get = get_wrapper
        return klass
    return _decorated


# -*- coding: utf-8 -*-
import json

from bunch import bunchify

from .base import ListResource, Resource
from .decorators import subresources

class CategoryResource(ListResource):

    resource_name = 'categories'


class CustomerResource(ListResource):

    resource_name = 'customers'


class OrderResource(ListResource):

    resource_name = 'orders'


@subresources(['variants', 'images'])
class ProductResource(ListResource):

    resource_name = 'products'


class ScriptResource(ListResource):

    resource_name = 'scripts'


class StoreResource(Resource):

    def get(self):
        """
        Get a single store.
        """
        return bunchify(json.loads(self._make_request('store').content))


class WebhookResource(ListResource):

    resource_name = 'webhooks'



# -*- coding: utf-8 -*-
import json
import logging

import requests
from furl import furl


logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.DEBUG)


def _do_verb(verb, url, payload, headers):
    params = {
        'url': url,
        'headers': headers
    }
    method = getattr(requests, verb)

    if verb in ['post', 'put']:
        params['headers']['Content-Type'] = 'application/json; charset=utf-8'
        params['data'] = json.dumps(payload)
    elif verb == 'get':
        params['params'] = payload

    return method(**params)


class APIClient(object):
    API_VERSION = 'v1'
    API_ENDPOINT = 'https://api.tiendanube.com'
    ARGS = ['resource_id', 'subresource', 'subresource_id']

    def __init__(self, api_key, user_agent):
        headers = {
            'Authentication': 'bearer {}'.format(api_key),
            'User-Agent': user_agent
        }
        self.headers = headers

    def get_options(self, args):
        return [args[k] for k in self.ARGS if k in args and args[k]]

    def make_request(self, id, resource, **kwargs):
        verb = kwargs.get('verb', 'GET').lower()

        url = furl(self.API_ENDPOINT)
        url.path.segments = [
            self.API_VERSION,
            id
        ]

        if resource:
            url.path.segments.append(resource)

        url.path.segments.extend(self.get_options(kwargs))

        payload = kwargs.get('extra') or kwargs.get('data')

        return _do_verb(verb, str(url), payload=payload, headers=self.headers)


# -*- coding: utf-8 -*-
from .api import APIClient
from .resources import (CategoryResource, CustomerResource,
                        OrderResource, ProductResource,
                        StoreResource, ScriptResource,
                        WebhookResource)


class Store(object):

    def __init__(self, http_client, store_id):
        self.store = StoreResource(http_client, store_id)
        self.customers = CustomerResource(http_client, store_id)
        self.products = ProductResource(http_client, store_id)
        self.categories = CategoryResource(http_client, store_id)
        self.orders = OrderResource(http_client, store_id)
        self.scripts = ScriptResource(http_client, store_id)
        self.webhooks = WebhookResource(http_client, store_id)

    def get_info(self):
        return self.store.get()


class NubeClient(object):

    def __init__(self, api_key, user_agent='MyNubeApp (mynubeapp.com)'):
        self._http_client = APIClient(api_key, user_agent)

    def get_store(self, store_id):
        return Store(self._http_client, str(store_id))



# -*- coding: utf-8 -*-
import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = 'tiendanube',
    packages = [
        'tiendanube',
        'tiendanube.resources',
    ],
    version = '1.1',
    description = '',
    license = 'MIT',
    long_description = read('README.rst'),
    url = 'https://github.com/catalanojuan/tiendanube-python',
    download_url = 'https://github.com/catalanojuan/tiendanube-python/tarball/1.1',


    author = 'Juan Catalano',
    author_email = 'catalanojuan@gmail.com',

    install_requires = [
        "argparse==1.2.1",
        "bunch==1.0.1",
        "furl==0.3.4",
        "ipython==1.0.0",
        "mock==1.0.1",
        "orderedmultidict==0.7.1",
        "py==1.4.15",
        "pytz==2013b",
        "requests==1.2.3",
        "wsgiref==0.1.2",
    ],

    classifiers = (
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ),
)



# -*- coding: utf-8 -*-
import datetime
import json

from bunch import bunchify

from .exceptions import APIError


def _get_value(val):
    if isinstance(val, datetime.datetime):
        return val.isoformat()
    return val


class Resource(object):

    def __init__(self, api_client, store_id):
        self.store_id = store_id
        self._http_client = api_client

    def _make_request(self, resource, **kwargs):
        response = self._http_client.make_request(self.store_id, resource, **kwargs)

        if response.status_code not in [200, 201]:
            raise APIError('{}. {}'.format(response.reason, response.text),
                           response.status_code)
        return response


class ListResource(Resource):

    def get(self, id):
        return bunchify(json.loads(self._make_request(self.resource_name, resource_id=str(id)).content))

    def list(self, filters={}, fields={}):
        """
        Get the list of customers for a store.
        """
        extra = {k:_get_value(v) for k,v in filters.items()}
        if fields:
            extra['fields'] = fields
        return bunchify(json.loads(self._make_request(self.resource_name, extra=extra).content))

    def add(self, resource_dict):
        return bunchify(json.loads(self._make_request(self.resource_name, data=resource_dict, verb='post').text))

    def update(self, resource_update_dict):
        res_id = str(resource_update_dict['id'])
        return bunchify(json.loads(self._make_request(self.resource_name, resource_id=res_id, data=resource_update_dict, verb='put').text))

class ListSubResource(ListResource):

    def __init__(self, resource, resource_id, subresource):
        super(ListSubResource, self).__init__(resource._http_client, resource.store_id)
        self.resource_name = resource.resource_name
        self.resource_id = resource_id
        self.subresource = subresource

    def get(self, id):
        return bunchify(json.loads(self._make_request(
            self.resource_name,
            resource_id=str(self.resource_id),
            subresource=self.subresource,
            subresource_id=str(id)).content)
        )

    def list(self, filters={}, fields={}):
        """
        Get the list of customers for a store.
        """
        extra = {k:_get_value(v) for k,v in filters.items()}
        if fields:
            extra['fields'] = fields
        return bunchify(json.loads(self._make_request(
            self.resource_name,
            resource_id=str(self.resource_id),
            subresource=self.subresource,
            extra=extra).content)
        )

    def add(self, subresource_dict):
        raise NotImplementedError('Sub resource add is not yet supported.')

    def update(self, subresource_update_dict):
        raise NotImplementedError('Sub resource update is not yet supported.')
